/*
 *	Network Queueing System (NQS)
 *  This version of NQS is Copyright (C) 1992  John Roman
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
*  PROJECT:     Network Queueing System
*  AUTHOR:      John Roman
*
*  Modification history:
*
*       Version Who     When            Description
*       -------+-------+---------------+-------------------------
*       V01.1   JRR     21-Sep-1992     Initial version.
*/
/*++ nqs_load.c - Network Queueing System
 *
 * $Source: /usr2/jrroma/nqs/nqs-3.30/src/RCS/nqs_load.c,v $
 *
 * DESCRIPTION:
 *
 *	NQS load module.
 *
 *
 *	Author:
 *	-------
 *	John Roman, Monsanto Company.
 *	September 21, 1992.
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision: 1.1 $ $Date: 1992/12/22 15:39:56 $ $State: Exp $)
 * $Log: nqs_load.c,v $
 * Revision 1.1  1992/12/22  15:39:56  jrroma
 * Version 3.30
 *
 * 
 *
 */
#include "nqs.h"
#include "nqsxvars.h"
#include "transactcc.h"		/* Transaction completion codes */
#include <malloc.h>

/*** nqs_load
 *
 *	Store the load information for this Mid in an internal data
 *	structure.
 *
 *	Returns:
 *		TCML_COMPLETE:	if successful;
 */
long nqs_load (mid, version, nqs_jobs, string)
int mid;
int version;
int nqs_jobs;
char *string;
{
    struct loadinfo *loadinfo_ptr;
    float load1,  load5,  load15;   
    
    if (Debug > 2)  {
        printf("D$Nqs_load: Entering with mid %d jobs %d \n", mid,  nqs_jobs);
        fflush (stdout);
    }
    sscanf(string,  "%f %f %f",  &load1,  &load5,  &load15);
    /*
     * Look through the queue of loadinfos for this mid.  If it is
     * there,  replace the contents with the new information.  If it is
     * not,  create a new structure.
     */
    loadinfo_ptr = Loadinfo_head.next;
    while (loadinfo_ptr != (struct loadinfo *) 0) {
	if (loadinfo_ptr->mid == mid) {
	    time (&loadinfo_ptr->time);
	    loadinfo_ptr->no_jobs = nqs_jobs;
	    loadinfo_ptr->load1 = load1;
	    loadinfo_ptr->load5 = load5;
	    loadinfo_ptr->load15 = load15;
	    break;
	} else {
	    loadinfo_ptr = loadinfo_ptr->next;
	}
    }
    if (loadinfo_ptr == (struct loadinfo *) 0) {
	loadinfo_ptr = (struct loadinfo *) malloc (sizeof (Loadinfo_head) );
	loadinfo_ptr->mid = mid;
	time (&loadinfo_ptr->time);
	loadinfo_ptr->no_jobs = nqs_jobs;
	loadinfo_ptr->load1 = load1;
	loadinfo_ptr->load5 = load5;
	loadinfo_ptr->load15 = load15;
	loadinfo_ptr->next = Loadinfo_head.next; 
	Loadinfo_head.next = loadinfo_ptr;
    }
	 
    return (TCML_COMPLETE);
}
