/*
 *	Network Queueing System (NQS)
 *  This version of NQS is Copyright (C) 1992  John Roman
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 1, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
*  PROJECT:     Network Queueing System
*  AUTHOR:      John Roman
*
*  Modification history:
*
*       Version Who     When            Description
*       -------+-------+---------------+-------------------------
*       V01.10  JRR     22-Jun-1992     Initial version.
*/
/*++ qcat.c - Network Deviceing System
 *
 * $Source: /usr2/jrroma/nqs/nqs-3.30/src/RCS/qcat.c,v $
 *
 * DESCRIPTION:
 *
 * RETURNS:
 *
 *      0       -  if output produced
 *      1       -  if an error occurred. A message is sent to the standard
 *                      output file for every error.
 *
 *
 *	Author:
 *	-------
 *	John Roman,  Monsanto Company
 *	June 22,  1992
 *
 *
 * STANDARDS VIOLATIONS:
 *   None.
 *
 * REVISION HISTORY: ($Revision: 1.1 $ $Date: 1992/12/22 15:43:00 $ $State: Exp $)
 * 
 *
 */

#define VERSION "V3.30 -- December,  1992"

#include "nqs.h"
#include <errno.h>
#include <string.h>
#if	HPUX | SGI | SYS52 | IBMRS | UNICOS | UTS
#include <fcntl.h>			/* File control */
#else
#if	BSD42 | BSD43 | ULTRIX
#include <sys/file.h>
#else
BAD SYSTEM TYPE
#endif
#endif
#include "nqsdirs.h"			/* NQS files and directories */

#ifndef __CEXTRACT__
#if __STDC__

static char *qcat_naminput ( long orig_seqno, Mid_t orig_mid );
static void qcat_showhow ( void );
static int show_version ( void );

#else /* __STDC__ */

static char *qcat_naminput (/* long orig_seqno, Mid_t orig_mid */);
static void qcat_showhow (/* void */);
static int show_version (/* void */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */


/*** main
 *
 * Copy a request's spooled input file, stdout, or stderr to stdout.
 * Format:
 *
 *	qcat [-e] [-i] [-o] [-v] seqno
 */
main (argc, argv)
int argc;
char *argv[];				/* [switch] & sequence number */
{
	struct {
		long orig_seqno;	/* Sequence# for req */
		Mid_t machine_id;	/* Machine-id of originating machine */
		Mid_t target_mid;       /* target mid */
	} req;				/* Request to do */
	char **scan_reqs;		/* Scan reqs */
	char *argument;			/* Ptr to cmd line arg text */
	Mid_t local_mid;		/* local machine-id */
	int show_stdout;		/* Flag to show stdout  */
	int show_stderr;		/* Flag to show stderr */
	int show_input;			/* Flag to show input file */
	struct rawreq rawreq;		/* Area to hold raw request structure */
	int c;				/* Character passing through */
	int rawreqfd;			/* Raw request file descriptor */
	char *path;			/* Relative path to the file of interest */
	FILE *fp;			/* File pointer to std[err,out] */
        int exitcode = 0;               /* Function exit code */

	if (chdir (Nqs_root) == -1) {
	    fprintf (stderr, "(FATAL): Unable to chdir() to the NQS ");
	    fprintf (stderr, "root directory.\n");
	    exit (1);
	}
	show_stdout = 1;		/* Assume want stdout */
        while (*++argv != NULL && **argv == '-') {
            argument = *argv;
            switch (*++argument) {
		case 'e':
		    show_stdout = 0;
		    show_stderr = 1;
		    show_input = 0;
		    break;
		case 'i':
		    show_stdout = 0;
		    show_stderr = 0;
		    show_input = 1;
		    break;
		case 'o':
		    show_stdout = 1;
		    show_stderr = 0;
		    show_input = 0;
		    break;
                case 'v':
                    show_version();
                    exit(0);
                default:
                    fprintf (stderr, "Invalid option flag ");
                    fprintf (stderr, "specified.\n");
                    qcat_showhow();
                    exit(0);
            }
        }
	if (*argv == NULL) {
	    /*
	     *  No request-ids were specified.
	     */
	    fprintf (stderr, "No request-id(s) specified.\n");
	    qcat_showhow();
	}
        scan_reqs = argv;		/* Set req scan pointer */
	switch (reqspec (*scan_reqs, &req.orig_seqno,
					 &req.machine_id,
					 &req.target_mid)) {
	    case -1:
		fprintf (stderr, "Invalid request-id syntax ");
	        fprintf (stderr, "for request-id: %s.\n",*scan_reqs);
	        exit (1);
	    case -2:
		fprintf (stderr, "Unknown machine for ");
		fprintf (stderr, "request-id: %s.\n",*scan_reqs);
	        exit (1);
	    case -3:
		fprintf (stderr, "Network mapping database ");
	        fprintf (stderr, "inaccessible.  Seek staff ");
	        fprintf (stderr, "support.\n");
	        exit (1);
	    case -4:
		fprintf (stderr, "Network mapping database ");
	        fprintf (stderr, "error when parsing ");
	        fprintf (stderr, "request-id: %s.\n",*scan_reqs);
	        fprintf (stderr, "Seek staff support.\n");
	        exit (1);
	}
        /* If reqspec returns null in machine id, force to
	 * local machine id.
	 */
	if (req.machine_id == 0) localmid(&req.machine_id);
	rawreqfd = gethdr(req.orig_seqno,  req.machine_id,  &rawreq);
	if (rawreqfd == -1) {
	    if (errno == ENOENT) {
		fprintf(stderr, "Request %s does not exist.\n",  *argv);
	    } else if (errno == EACCES) {
		fprintf(stderr,  "No access to request %s.\n",  *argv);
	    } else {
		fprintf(stderr, "Error reading header\n");
	        perror(" ");
	    }
	    fflush(stdout);
	    exit(0);
	}
        /*
         *  Generate name of temporary spooled output file.
         */
	if (show_stdout)
	        path = namstdout (rawreq.orig_seqno, rawreq.orig_mid);
	else if (show_stderr)
	        path = namstderr (rawreq.orig_seqno, rawreq.orig_mid);
	else if (show_input)
		path = qcat_naminput (rawreq.orig_seqno,  rawreq.orig_mid);
	if ( (fp = fopen (path, "r")) == NULL) {
	    if (errno == ENOENT) {
		if (show_stdout) fprintf(stderr,  "Output file does not exist.\n");
		else if (show_stderr) fprintf(stderr, "Error file does not exist.\n");
		else fprintf(stderr,  "Input file does not exist.\n");
		exit(0);
	    }
	    fprintf(stderr,  "Error opening %s\n",  path);
	    perror(" ");
	    exit(1);
	}
	
	while (1) {
	    c = getc(fp);
	    if (c == EOF) break;
	    putc(c,  stdout);	    
	}
	/*
	 *  Flush output buffers and exit.
	 */
	fflush (stdout);
	fflush (stderr);
	exit (exitcode);
}
/*** qcat_naminput
 *
 *
 *      char *qcat_naminput():
 *
 *      Return pointer to name of spooled stdout output file for
 *      a batch request.
 *
 *      NOTE:   The name is computed relative to the NQS root directory.
 */
static char *qcat_naminput (orig_seqno, orig_mid)
long orig_seqno;                        /* Req sequence number */
Mid_t orig_mid;                         /* Req machine-id */
{
        static char path [48];          /* Pathname for file.  This size */
                                        /* may need to be increased.... */

        pack6name (path, Nqs_data, (int) (orig_seqno % MAX_DATASUBDIRS),
                  (char *) 0, (long) orig_seqno, 5, (long) orig_mid, 6, 0, 3);
        return (path);
}

/*** qcat_showhow
 *
 *
 *	qcat_showhow():
 *	Show how to use this command.
 */
static void qcat_showhow()
{
	fprintf (stderr, "Command format is:\n\n");
	fprintf (stderr, "    qcat  [-e] [-i] [-o] [-v] <request-id>\n\n");
	exit (0);
}
static show_version()
{
	fprintf (stderr, "Qcat version is %s.\n", VERSION);
}
